"""A Bokeh model for the JSMEEditor"""
from bokeh.core.properties import List, String
from bokeh.models import HTMLBox
from panel import extension

# pylint: disable=protected-access
extension._imports["jsme"] = "panel_chemistry.bokeh_extensions.jsme_editor"
# pylint: enable=protected-access


class JSMEEditor(HTMLBox):
    """JSMEEditor Bokeh Model"""

    __javascript__ = ["https://unpkg.com/jsme-editor@0.8.2/jsme.nocache.js"]

    value = String()
    format = String()
    subscriptions = List(String())
    options = List(String())

    jme = String()
    smiles = String()
    mol = String()
    mol3000 = String()
    sdf = String()

    guicolor = String()



"""A Bokeh Model of the NGLViewer"""
from bokeh.core.properties import List, String
from bokeh.models import LayoutDOM


class NGLViewer(LayoutDOM):
    """The [NGL Viewer](https://github.com/nglviewer/ngl) can be used
    to show and analyse pdb molecule structures"""

    object = String()
    extension = String()
    representation = String()
    color_scheme = String()
    effect = String()
    custom_color_scheme = List(List(String))

    __javascript__ = [
        "https://unpkg.com/ngl@2.0.0-dev.37/dist/ngl.js",
    ]

    __js_skip__ = {
        "NGL": __javascript__[:1],
    }

    __js_require__ = {
        "paths": {
            "NGL": "https://unpkg.com/ngl@2.0.0-dev.37/dist/ngl",
        },
        "exports": {"NGL": "NGL"},
    }



"""The [NGL Viewer](https://github.com/nglviewer/ngl) can be used
to visualize and analyse pdb molecule structures.

Source: Discussion on [Discourse 583]\
(https://discourse.holoviz.org/t/how-to-use-ngl-webgl-protein-viewer-in-panel/583)

Author: https://github.com/Jhsmit
"""

from typing import Dict

import param
from panel import extension
from panel.pane.base import PaneBase
from panel.util import lazy_load, string_types
from pyviz_comms import JupyterComm

# pylint: disable=protected-access
extension._imports["ngl_viewer"] = "panel_chemistry.bokeh_extensions.ngl_viewer"
# pylint: enable=protected-access

REPRESENTATIONS = [
    # "base",
    # "distance",
    "axes",
    "backbone",
    "ball+stick",
    "cartoon",
    "helixorient",
    "hyperball",
    "label",
    "licorice",
    "line",
    "point",
    "ribbon",
    "rocket",
    "rope",
    "spacefill",
    "surface",
    "trace",
    "unitcell",
    # "validation",
]
COLOR_SCHEMES = [
    "atomindex",
    "bfactor",
    "chainid",
    "chainindex",
    "chainname",
    "custom",
    "densityfit",
    "electrostatic",
    "element",
    "entityindex",
    "entitytype",
    "geoquality",
    "hydrophobicity",
    "modelindex",
    "moleculetype",
    "occupancy",
    "random",
    "residueindex",
    "resname",
    "sstruc",
    "uniform",
    "value",
    "volume",
]
EXTENSIONS = [
    "",
    "pdb",
    "cif",
    "csv",
    "ent",
    "gro",
    "json",
    "mcif",
    "mmcif",
    "mmtf",
    "mol2",
    "msgpack",
    "netcdf",
    "parm7",
    "pqr",
    "prmtop",
    "psf",
    "sd",
    "sdf",
    "top",
    "txt",
    "xml",
]


class NGLViewer(PaneBase):
    """The [NGL Viewer](https://github.com/nglviewer/ngl) can be used
    to show and analyse pdb molecule structures"""

    object = param.String(
        doc="""
        The object to display. For example an url like 'rcsb://3dqb.pdb', 'rcsb://1NKT', '1NKT'.
        You can also specify a extension string if you define the extension 
        in the extension parameter"""
    )
    extension = param.ObjectSelector(
        default="",
        objects=EXTENSIONS,
        doc="""
        If you specify a extension extension != '', then the object will be loaded as a blob string .
        Default is '', i.e. the object is loaded as a url.""",
    )
    representation = param.Selector(
        default="ball+stick",
        objects=REPRESENTATIONS,
        doc="""
        A display representation. Default is 'ball+stick'. See
        http://nglviewer.org/ngl/api/manual/coloring.html#representations
        """,
    )
    color_scheme = param.Selector(
        default="element",
        objects=COLOR_SCHEMES,
        doc="""
    A predefined or 'custom' color scheme. If 'custom' is specified you need to specify the
    'custom_color_scheme' parameter. Default is 'element'. See
    http://nglviewer.org/ngl/api/manual/coloring.html""",
    )
    custom_color_scheme = param.List(
        default=[["white", "*"]],
        doc="""
    A custom color scheme. See
    http://nglviewer.org/ngl/api/manual/coloring.html#custom-coloring.""",
    )
    effect = param.Selector(
        default="",
        objects=["", "spin", "rock"],
        doc="""
    One of '', 'spin' or 'rock'. Default is '', i.e. no effect""",
    )

    priority = None

    _rename: Dict[str, str] = {}

    _updates = True

    _model_module = "panel_chemistry.bokeh_extensions.ngl_viewer"
    _model = "NGLViewer"

    @classmethod
    def applies(cls, obj):
        if isinstance(obj, (dict, string_types)):
            return 0
        return False

    def _get_model(self, doc, root=None, parent=None, comm=None):
        _NGLViewer = lazy_load(  # pylint: disable=invalid-name
            self._model_module, self._model, isinstance(comm, JupyterComm)
        )
        props = self._process_param_change(self._init_params())
        if not "object" in props:
            props["object"] = self.object
        model = _NGLViewer(**props)
        root = root or model
        # self._link_props(
        #     model,
        #     [
        #         "event",
        #     ],
        #     doc,
        #     root,
        #     comm,
        # )
        self._models[root.ref["id"]] = (model, parent)
        return model

    def _update(self, ref=None, model=None):
        model.update(
            object=self.object,
            extension=self.extension,
            representation=self.representation,
            color_scheme=self.color_scheme,
            effect=self.effect,
            custom_color_scheme=self.custom_color_scheme,
        )



"""A Panel Pane to wrap the PDBe implementation of the Mol* ('MolStar') viewer.

Check out

- [PDBe Mol*](https://github.com/PDBeurope/pdbe-molstar)
- [Mol*](https://molstar.org/)
- [Mol* GitHub](https://github.com/molstar/molstar)

Cite Mol*:
David Sehnal, Sebastian Bittrich, Mandar Deshpande, Radka Svobodová, Karel Berka,
Václav Bazgier, Sameer Velankar, Stephen K Burley, Jaroslav Koča, Alexander S Rose:
Mol* Viewer: modern web app for 3D visualization and analysis of large biomolecular structures,
Nucleic Acids Research, 2021; https://doi.org/10.1093/nar/gkab314.
"""

import param
from panel.reactive import ReactiveHTML

REPRESENTATIONS = [
    "cartoon",
    "ball-and-stick",
    "carbohydrate",
    "distance-restraint",
    "ellipsoid",
    "gaussian-surface",
    "molecular-surface",
    "point",
    "putty",
    "spacefill",
]

# See https://embed.plnkr.co/plunk/m3GxFYx9cBjIanBp for an example JS implementation
class PDBeMolStar(ReactiveHTML):
    """PDBe MolStar structure viewer.

    Set one of `molecule_id`, `custom_data` and `ligand_view`.

    For more information:

    - https://github.com/PDBeurope/pdbe-molstar/wiki
    - https://molstar.org/


    The implementation is based on the JS Plugin. See
    - https://github.com/PDBeurope/pdbe-molstar/wiki/1.-PDBe-Molstar-as-JS-plugin
    For documentation on the helper methods:
    - https://github.com/molstar/pdbe-molstar/wiki/3.-Helper-Methods

    """

    molecule_id = param.String(default=None, doc="PDB id to load. Example: '1qyn' or '1cbs'")

    custom_data = param.Dict(
        doc="""Load data from a specific data source. Example:
        { "url": "https://www.ebi.ac.uk/pdbe/coordinates/1cbs/chains?entityId=1&asymId=A&encoding=bcif", "format": "cif", "binary": True }
        """
    )
    ligand_view = param.Dict(
        doc="""This option can be used to display the PDBe ligand page 3D view
        like https://www.ebi.ac.uk/pdbe/entry/pdb/1cbs/bound/REA.
        Example: {"label_comp_id": "REA"}
        """
    )

    alphafold_view = param.Boolean(
        default=False, doc="Applies AlphaFold confidence score colouring theme for alphafold model"
    )

    assembly_id = param.String(doc="Specify assembly")

    bg_color = param.Color(
        "#F7F7F7",
        doc="""Color of the background. If `None`, colors default is chosen
        depending on the color theme""",
    )

    highlight_color = param.Color(default="#ff6699", doc="Color for mouseover highlighting")

    select_color = param.Color(default="#0c0d11", doc="Color for selections")

    visual_style = param.Selector(
        default=None, objects=[None, *REPRESENTATIONS], doc="Visual styling"
    )

    theme = param.Selector(default="default", objects=["default", "dark"], doc="CSS theme to use")

    hide_polymer = param.Boolean(default=False, doc="Hide polymer")

    hide_water = param.Boolean(default=False, doc="Hide water")

    hide_heteroatoms = param.Boolean(default=False, doc="Hide het")

    hide_carbs = param.Boolean(default=False, doc="Hide carbs")

    hide_non_standard = param.Boolean(default=False, doc="Hide non standard")

    hide_coarse = param.Boolean(default=False, doc="Hide coarse")

    hide_controls_icon = param.Boolean(default=False, doc="Hide the control menu")

    hide_expand_icon = param.Boolean(default=False, doc="Hide the expand icon")

    hide_settings_icon = param.Boolean(default=False, doc="Hide the settings menu")

    hide_selection_icon = param.Boolean(
        default=False, doc="Hide the selection icon"  # Default False, set False/True for True
    )

    hide_animation_icon = param.Boolean(default=False, doc="Hide the animation icon")

    pdbe_url = param.String(
        default=None, constant=True, doc="Url for PDB data. Mostly used for internal testing"
    )

    load_maps = param.Boolean(
        default=False, doc="Load electron density maps from the pdb volume server"
    )

    validation_annotation = param.Boolean(
        default=False, doc="Adds 'annotation' control in the menu"
    )

    domain_annotation = param.Boolean(default=False, doc="Adds 'annotation' control in the menu")

    low_precision_coords = param.Boolean(
        default=False, doc="Load low precision coordinates from the model server"
    )

    hide_controls = param.Boolean(default=True, doc="Hide the control menu")

    expanded = param.Boolean(default=False, doc="""Display full-screen by default on load""")

    landscape = param.Boolean(
        default=True,  # Changed to True because it works best with Panel currently
        doc="""Set landscape view. The controls will similar to the full-screen view""",
    )

    select_interaction = param.Boolean(
        default=True, doc="Switch on or off the default selection interaction behaviour"
    )

    lighting = param.Selector(
        default="matte",
        objects=["flat", "matte", "glossy", "metallic", "plastic"],
        doc="Set the lighting",
    )

    default_preset = param.Selector(
        default="default",
        objects=["default", "unitcell", "all-models", "supercell"],
        doc="Set the preset view",
    )

    pdbe_link = param.Boolean(
        default=True, doc="Show the PDBe entry link at in the top right corner"
    )

    spin = param.Boolean(default=False, doc="Toggle spin")

    _clear_highlight = param.Boolean(doc="Event to trigger clearing of highlights")

    _select = param.Dict(doc="Dictionary used for selections and coloring these selections")

    _clear_selection = param.Boolean(doc="Clear selection event trigger")

    _highlight = param.Dict(doc="Dictionary used for selections and coloring these selections")

    _reset = param.Boolean(doc="Reset event trigger")

    _args = param.Dict(doc="Dictionary with function call arguments")

    test = param.Boolean(default=False)

    _template = """
<link id="molstarTheme" rel="stylesheet" type="text/css" href="https://www.ebi.ac.uk/pdbe/pdb-component-library/css/pdbe-molstar-1.2.1.css"/>
<div id="container" style="width:100%; height: 100%;"><div id="pdbeViewer"></div></div>
"""
    __javascript__ = [
        "https://www.ebi.ac.uk/pdbe/pdb-component-library/js/pdbe-molstar-plugin-1.2.1.js",
    ]

    _scripts = {
        "render": r"""
        function standardize_color(str){
            var ctx = document.createElement("canvas").getContext("2d");
            ctx.fillStyle = str;
            return ctx.fillStyle;
        }
        function toRgb(color) {
          var hex = standardize_color(color)
          var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
          return result ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16)
          } : null;
        }
        state.toRgb = toRgb
        
        function getHideStructure(){
            var hideStructure = [];
        
            if (data.hide_polymer){
                hideStructure.push("polymer")
            }
            if (data.hide_water){
                hideStructure.push("water")
            }
            if (data.hide_heteroatoms){
                hideStructure.push("het")
            }
            if (data.hide_carbs){
                hideStructure.push("carbs")
            }
            if (data.hide_non_standard){
                hideStructure.push("nonStandard")
            }
            if (data.hide_coarse){
                hideStructure.push("coarse")
            }
        
            return hideStructure
        }
        
        function getHideCanvasControls(){
            var hideCanvasControls = [];
            if (data.hide_controls_icon){
                hideCanvasControls.push("controlToggle")
            }
            if (data.hide_expand_icon){
                hideCanvasControls.push("expand")
            }
            if (data.hide_settings_icon){
                hideCanvasControls.push("controlInfo")
            }
            if (data.hide_selection_icon){
                hideCanvasControls.push('selection')
            }
            if (data.hide_animation_icon){
                hideCanvasControls.push("animation")
            }
        
            return hideCanvasControls
        }
        
        state.getHideCanvasControls = getHideCanvasControls
        
        function getOptions(){
            var options = {
                moleculeId: data.molecule_id,
                customData: data.custom_data,
                ligandView: data.ligand_view,
                alphafoldView: data.alphafold_view,
                assemblyId: data.assembly_id,
                bgColor: toRgb(data.bg_color),
                highlightColor: toRgb(data.highlight_color),
                selectColor: toRgb(data.select_color),
                hideStructure: getHideStructure(),
                hideCanvasControls: getHideCanvasControls(),
                loadMaps: data.load_maps,
                validationAnnotation: data.validation_annotation,
                domainAnnotation: data.domain_annotation,
                lowPrecisionCoords: data.low_precision_coords,
                expanded: data.expanded,
                hideControls: data.hide_controls,
                landscape: data.landscape,
                selectInteraction: data.select_interaction,
                lighting: data.lighting,
                defaultPreset: data.default_preset,
                pdbeLink: data.pdbe_link,
            }
            if (data.visual_style!==null){
                options["visualStyle"]=data.visual_style
            }
            if (data.pdbe_url!==null){
                options["pdbeUrl"]=data.pdbe_url
            }
            return options
        }
        state.getOptions=getOptions
        self.theme()
        
        state.viewerInstance = new PDBeMolstarPlugin();
        state.viewerInstance.render(pdbeViewer, state.getOptions());
        
        
        """,
        "rerender": """
        state.viewerInstance.visual.update(state.getOptions(), fullLoad=true)
        """,
        "molecule_id": """self.rerender()""",
        "custom_data": """self.rerender()""",
        "ligand_view": """self.rerender()""",
        "alphafold_view": """self.rerender()""",
        "assembly_id": """self.rerender()""",
        "visual_style": """self.rerender()""",
        "bg_color": "state.viewerInstance.canvas.setBgColor(state.toRgb(data.bg_color))",
        "highlight_color": """
        state.viewerInstance.visual.setColor({highlight: state.toRgb(data.highlight_color)})""",
        "select_color": """
        state.viewerInstance.visual.setColor({select: state.toRgb(data.select_color)})""",
        "theme": """
        if (data.theme==="dark"){
            molstarTheme.href="https://www.ebi.ac.uk/pdbe/pdb-component-library/css/pdbe-molstar-1.2.1.css"
        } else {
            molstarTheme.href="https://www.ebi.ac.uk/pdbe/pdb-component-library/css/pdbe-molstar-light-1.2.1.css"
        }
        """,
        "hide_polymer": "state.viewerInstance.visual.visibility({polymer:!data.hide_polymer})",
        "hide_water": "state.viewerInstance.visual.visibility({water:!data.hide_water})",
        "hide_heteroatoms": "state.viewerInstance.visual.visibility({het:!data.hide_heteroatoms})",
        "hide_carbs": "state.viewerInstance.visual.visibility({carbs:!data.hide_carbs})",
        # pylint: disable=line-too-long
        "hide_non_standard": "state.viewerInstance.visual.visibility({nonStandard:!data.hide_non_standard})",
        # pylint: enable=line-too-long
        "hide_coarse": "state.viewerInstance.visual.visibility({coarse:!data.hide_coarse})",
        "hide_controls_icon": """self.rerender()""",
        "hide_expand_icon": """self.rerender()""",
        "hide_settings_icon": """self.rerender()""",
        "hide_selection_icon": """self.rerender()""",
        "hide_animation_icon": """self.rerender()""",
        "load_maps": "self.rerender()",
        "validation_annotation": """self.rerender()""",
        "domain_annotation": """self.rerender()""",
        "low_precision_coords": """self.rerender()""",
        "expanded": "state.viewerInstance.canvas.toggleExpanded(data.expanded)",
        "landscape": """self.rerender()""",
        "select_interaction": """self.rerender()""",
        "lighting": """self.rerender()""",
        "default_preset": """self.rerender()""",
        "pdbe_link": """self.rerender()""",
        "hide_controls": "state.viewerInstance.canvas.toggleControls(!data.hide_controls);",
        "spin": """state.viewerInstance.visual.toggleSpin(data.spin);""",
        "_select": """
        if(data._select) {
        state.viewerInstance.visual.select(data._select);
        }
        """,
        "_clear_selection": """
        state.viewerInstance.visual.clearSelection(data._args['number']);
        """,
        "_highlight": """
        if(data._highlight) {
            state.viewerInstance.visual.highlight(data._highlight);
        };           
        """,
        "_clear_highlight": """
        state.viewerInstance.visual.clearHighlight();
        """,
        "_reset": """
        state.viewerInstance.visual.reset(data._args['data'])""",
        "resize": "state.viewerInstance.canvas.handleResize()",
    }

    def color(self, data, non_selected_color=None):
        """
        Alias for PDBE Molstar's `select` method.

        See https://github.com/molstar/pdbe-molstar/wiki/3.-Helper-Methods for parameter
        details

        :param data: List of dicts
        :param non_selected_color: Dict of color example: {'r':255, 'g':215, 'b': 0}
        :return: None
        """

        self._select = {"data": data, "nonSelectedColor": non_selected_color}
        self._select = None

    def clear_selection(self, structure_number=None):
        """
        Clear selection

        See https://github.com/molstar/pdbe-molstar/wiki/3.-Helper-Methods for parameter
        details.

        :param structure_number: Optional integer to specify structure number
        :return:
        """

        self._args = {"number": structure_number}
        self._clear_selection = not self._clear_selection

    def highlight(self, data):
        """
        Trigger highlight

        See https://github.com/molstar/pdbe-molstar/wiki/3.-Helper-Methods for parameter
        details.

        :param data: List of dicts
        :return: None
        """

        self._highlight = {"data": data}
        self._highlight = None

    def clear_highlight(self):
        """Clears the current highlight"""
        self._clear_highlight = not self._clear_highlight

    def reset(self, data):
        """
        Reset to defaults

        See https://github.com/molstar/pdbe-molstar/wiki/3.-Helper-Methods for parameter
        details.

        :param data: Dictionary of options to reset to defaults
        :return:
        """

        self._args = {"data": data}
        self._reset = not self._reset



"""The panel-chemistry JSME Molecule Editor enables you to draw and edit molecules using Python
and Panel.

It is a wrapper of the free javascript JSME Molecule Editor. See https://jsme-editor.github.io/
"""
import param
from panel.widgets.base import Widget

from panel_chemistry.bokeh_extensions.jsme_editor import JSMEEditor as _BkJSMEEditor

NOT_SUBSCRIBED = "Not Subscribed"
VALUE_FORMATS = ["jme", "smiles", "mol", "mol3000", "sdf"]
# fmt: off
# Defaults can be found here http://wiki.jmol.org/index.php/Jmol_JavaScript_Object/JME/Options
OPTIONS = [
    "addnewpart", "noaddnewpart",
    "atommovebutton", "noatommovebutton",
    "autoez", "noautoez",
    "border", "noborder",
    "canonize", "nocanonize",
    "contextmenu", "nocontextmenu",
    "depict", "nodepict",
    "depictaction", "nodepictaction",
    "exportinchi", "noexportinchi",
    "exportinchiauxinfo", "noexportinchiauxinfo",
    "exportinchikey", "noexportinchikey",
    "exportSVG", "noexportSVG",
    "fgmenu", "nofgmenu",
    "fullScreenIcon", "nofullScreenIcon",
    "hydrogens", "nohydrogens",
    "keephs", "removehs",
    "keephsc", "removehsc",
    "markAtomOnly", "noMarkAtomOnly",
    "markBondOnly", "noMarkBondOnly",
    "marker", "nomarker",
    "marker1", "nomarker1",
    "markNothing", "noMarkNothing",
    "multipart", "nomultipart",
    "newlook", "oldlook",
    "number", "autonumber",
    "paste", "nopaste",
    "polarnitro", "nopolarnitro",
    "pseudoMark", "nopseudoMark",
    "query", "noquery",
    "rbutton", "norbutton",
    "reaction", "noreaction",
    "searchinchiKey", "nosearchinchiKey",
    "showdragandDropIconindepictmode", "noShowdragandDropIconindepictmode",
    "showFullScreenIconInDepictMode", "noshowFullScreenIconInDepictMode",
    "stereo", "nostereo",
    "toggle", "notoggle",
    "useOclidcode", "nouseOclidcode",
    "useopenchemlib", "nouseopenchemlib",
    "valenceState", "noValenceState",
    "xbutton", "noxbutton",
    "zoom", "nozoom",
]
# fmt: on


class JSMEEditor(Widget):
    """The panel-chemistry JSME Molecule Editor enables you to draw and edit molecules using Python
    and Panel.
    It is a wrapper of the free javascript JSME Molecule Editor.
    See https://jsme-editor.github.io/"""

    # Set the Bokeh model to use
    _widget_type = _BkJSMEEditor

    # Rename Panel Parameters -> Bokeh Model properties
    # Parameters like title that does not exist on the Bokeh model should be renamed to None
    _rename = {
        "title": None,
    }

    # Parameters to be mapped to Bokeh model properties
    value = param.String(
        default="",
        doc="""A value definining the structure of the molecule. The
    value provided from Python can be any of the available `format` values. The value returned
    from the client will be in the specified `format` value.""",
    )
    format = param.ObjectSelector(
        "jme",
        objects=VALUE_FORMATS,
        doc="""
    The format of the structure returned from the client. Can be any of "jme" (default), "smiles",
    "mol", "mol3000", "sdf\"""",
    )
    # Note: Could be implemented as a child class of booleans/ checkboxes instead.
    options = param.ListSelector(
        objects=OPTIONS,
        doc="""A list of options to apply to the editor.
    Default is [], i.e. to use the default settings.""",
    )

    subscriptions = param.ListSelector(
        default=[],
        objects=VALUE_FORMATS,
        doc="""
    A list of structure formats to "subscribe" to changes of. Can be any of "jme", "smiles",
    "mol", "mol3000", "sdf". Default is [].""",
    )

    jme = param.String(
        default=NOT_SUBSCRIBED,
        constant=True,
        doc="""
    The structure in jme format""",
    )
    smiles = param.String(
        default=NOT_SUBSCRIBED,
        constant=True,
        doc="""
    The structure in smiles format""",
    )
    mol = param.String(
        default=NOT_SUBSCRIBED,
        constant=True,
        doc="""
    The structure in mol format""",
    )
    mol3000 = param.String(
        default=NOT_SUBSCRIBED,
        constant=True,
        doc="""
    The structure in mol3000 format""",
    )
    sdf = param.String(
        default=NOT_SUBSCRIBED,
        constant=True,
        label="SDF",
        doc="""
    The structure in sdf format""",
    )

    guicolor = param.Color(
        default="#c0c0c0",
        doc="""
    Background color of the GUI elements in RGB hex format. Default is "#c0c0c0".""",
    )



"""A Panel Pane to wrap the interactive py3Dmol/ 3Dmol.js viewer in your Panel Application.

Check out

- [3DMol.js](https://3dmol.org/index.html)
- [Colab](https://colab.research.google.com/drive/1T2zR59TXyWRcNxRgOAiqVPJWhep83NV_?usp=sharing)
"""

import panel as pn
import param

try:
    import py3Dmol
except ModuleNotFoundError:
    # pylint: disable=invalid-name
    class py3Dmol(param.Parameterized):  # type: ignore
        """Dummy py3Dmol class"""
        view = param.String('mock_view')  # allow to import the file


def _clean_html(html):
    """Returns the py3Dmol.view._make_html with 100% height and width"""
    start = html.find("width:")
    end = html.find('px">') + 2
    size_str = html[start:end]
    html = html.replace(size_str, "width: 100%; height: 100%")

    return html


class Py3DMol(pn.viewable.Viewer):
    """A Panel Pane to wrap the interactive py3Dmol/ 3Dmol.js viewer in your Panel Application.

    Check out

    - [3DMol.js](https://3dmol.org/index.html)
    - [Colab](https://colab.research.google.com/drive/1T2zR59TXyWRcNxRgOAiqVPJWhep83NV_?usp=sharing)
    """

    object: py3Dmol.view = param.ClassSelector(
        class_=py3Dmol.view,
        doc="""
    A py3Dmol.view
    """,
    )

    layout = param.ClassSelector(
        class_=pn.pane.HTML,
        constant=True,
        doc="""
    A HTML pane containing the py3Dmol.view html. You can apply all the usual layout parameters to
    this layout.
    """,
    )

    def __init__(self, object: py3Dmol.view = None, **params): # pylint: disable=redefined-builtin
        """A Panel Pane to wrap the interactive py3Dmol/ 3Dmol.js viewer in your Panel Application.

        Check out

        - [3DMol.js](https://3dmol.org/index.html)
        - [Colab]\
(https://colab.research.google.com/drive/1T2zR59TXyWRcNxRgOAiqVPJWhep83NV_?usp=sharing)

        Args:
            object (py3Dmol.view, optional): A py3Dmol.view. Defaults to None.
            params (): These parameters are applied to the layout. Could be height, width etc.
        """
        super().__init__(object=object, layout=pn.pane.HTML(**params))
        if "name" in params:
            with param.edit_constant(self):
                self.name = params["name"]
        self._update_name()
        self._update_layout()

    @pn.depends("name", watch=True)
    def _update_name(self):
        with param.edit_constant(self.layout):
            self.layout.name = self.name

    @pn.depends("object", watch=True)
    def _update_layout(self):
        if self.object:
            # pylint: disable=no-member, protected-access
            self.layout.object = _clean_html(self.object._make_html())
        else:
            self.layout.object = ""

    def __panel__(self):
        return self.layout
